<?php

namespace Drupal\onlyone;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Language\LanguageManagerInterface;

/**
 * Class OnlyOne.
 *
 * @package Drupal\onlyone
 */
class OnlyOne {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The Drupal\Core\Language\LanguageManagerInterface definition.
   *
   * @var Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Connection $connection, LanguageManagerInterface $language_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->connection = $connection;
    $this->languageManager = $language_manager;
  }

  /**
   * Return a content type label list.
   *
   * @return array
   *   An associative array with the content type machine name as key
   *   and his label as value.
   */
  public function getContentTypesList() {
    // Looking for all the content types, as the content types are only
    // accesibles through the entity_type.manager service we can't use a
    // LEFT JOIN in only one query as is posible in Drupal 7
    // (See Drupal 7 module version) read mode about this at:
    // http://drupal.stackexchange.com/q/233214/28275 .
    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    // Creating an array with all content types.
    $content_types_list_label = [];
    foreach ($content_types as $content_type) {
      $content_types_list_label[$content_type->id()] = $content_type->label();
    }

    return $content_types_list_label;
  }

  /**
   * Return the temporary table name created with all the content types names.
   *
   * @return string
   *   The temporary table name.
   */
  public function getTemporaryContentTypesTable() {
    // Looking for all the content types, as the content types are only
    // accesibles through the entity_type.manager service we can't use a
    // LEFT JOIN in only one query as is posible in Drupal 7
    // (See Drupal 7 module version) read mode about this at:
    // http://drupal.stackexchange.com/q/233214/28275 and
    // https://drupal.stackexchange.com/q/235640/28275 .
    // Looking for the content type names from the config table.
    $query_content_types = "SELECT DISTINCT SUBSTR(name, 11) AS type
                          FROM {config}
                          WHERE name LIKE 'node.type.%'";
    // Creating a temporary table and returning the name.
    return $this->connection->queryTemporary($query_content_types);
  }

  /**
   * Return if exists nodes of a content type in the actual language.
   *
   * @param string $content_type
   *   Content type machine name.
   *
   * @return int
   *   If exists nodes return the first node nid otherwise return 0.
   */
  public function existsNodesContentType($content_type) {
    $query = \Drupal::entityQuery('node');
    $query->condition('type', $content_type);
    // The site is multilingual?
    if ($this->languageManager->isMultilingual()) {
      // @TODO: Check if this will work for Multilingual sites.
      $language = $this->languageManager->getCurrentLanguage()->getId();
      $query->condition('langcode', $language);
    }
    $nid = $query->execute();
    if (count($nid)) {
      // Extracting the nid from the array.
      $nid = array_pop($nid);
    }
    else {
      $nid = 0;
    }
    return $nid;
  }

}
